/***********************************************************************

	Copyright (c) 1994 Wesley Steiner o/a KW Software
	All rights reserved.

***********************************************************************/

/***********************************************************************

Purpose:

	Program to test the PCL parser.

Syntax:

	PCL2TXT <file> [<stdout>]

Input:

	HP PCL file.

Output:

	A formatted listing to stdout of the PCL commands and data
	in the input.

Notes:

$Header:   W:/ljeng/pcl2txt/pcl2txt.c_v   1.6   14 Sep 1994 10:04:02   WES  $

$Log:   W:/ljeng/pcl2txt/pcl2txt.c_v  $
 * 
 *    Rev 1.6   14 Sep 1994 10:04:02   WES
 *  
 * 
 *    Rev 1.5   07 Sep 1994  7:32:06   WES
 * replaced entry vector with func calls
 * 
 *    Rev 1.4   06 Sep 1994 15:36:48   WES
 * moved to seperate dir
 * 
 *    Rev 1.3   06 Sep 1994 10:36:10   WES
 * generate 8086 compatible code
 *
 *    Rev 1.2   06 Sep 1994  9:40:54   WES
 *  
 * 
 *    Rev 1.1   28 Aug 1994  8:50:34   WES
 *  
 * 
 *    Rev 1.0   28 Aug 1994  8:37:24   WES
 * Initial revision.
 *
 *    Rev 1.0   20 May 1994 14:16:14   WES
 * Initial revision.

***********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <sys\stat.h>

#include "wes.h"

#include "pcllib.h"

#define VERSION "1.0.2"

int was_data = 0; /* last byte was a data byte */
int hex_mode = 0; /* currently consuming bytes of data */
int db_count = 0; /* counts # of chr/hex bytes output on a line */

char _pchar, _gchar;

void NewLine()
	{
	/*if (was_data && !hex_mode) printf("\"");*/
	printf("\n");
	db_count = 0;
	}

int PCL_Hook_ESC(void)
	{
	NewLine();
	was_data = 0;
	hex_mode = db_count = 0;
	return 0;
	}

int PCL_Hook_PCH(const char PCH)
	{
	_pchar = PCH;
	printf("!%c", PCH);
	return 0;
	}

int PCL_Hook_GCH(const char GCH)
	{
	_gchar = GCH;
	printf("%c", GCH);
	return 0;
	}

int _datacnt_; /* bytes of raw data to read for the current command */
int _cnt_;

void HexOut(const unsigned char db)
	{
	if (!was_data || (was_data && !hex_mode)) NewLine();
	was_data = 1;
	if (db_count == 16) NewLine();
	hex_mode = 1;
	printf("%02X ", db);
	++db_count;
	}

void ChrOut(const unsigned char db)
	{
	if (!was_data || hex_mode) NewLine();
	was_data = 1;
	/*if (!db_count) printf("\"");*/
	hex_mode = 0;
	printf("%c", db);
	++db_count;
	}

int GetData(const unsigned char data)
	{
	if (--_cnt_ == 0) PCL_Entry(PCL_Default);
	HexOut(data);
	return 0;
	}

int CollectData(void)
	{
	if (!was_data) NewLine();
	was_data = 1;
	hex_mode = 1;
	if (_datacnt_ == 0)
		{
		PCL_Entry(PCL_Default);
		return 0;
		}
	_cnt_ = _datacnt_;
	PCL_Entry(GetData);
	return 0;
	}

int PCL_Hook_TCH(const char *Val, const char TCH)
	{
	printf("%s%c", Val, TCH);
	/*if (isupper(TCH)) NewLine();*/
	_datacnt_ = abs(atoi(Val));
	switch(_pchar) {
		case '&':
			switch(_gchar) {
				case 'p':
					switch(TCH) {
						case 'X': /* transparent print data */
							return CollectData();
						}
				}
		case '(':
		case ')':
			switch(_gchar) {
				case 's':
					switch(TCH) {
						case 'W': /* download character */
							return CollectData();
						}
				}
		case '*':
			switch(_gchar) {
				case 'b':
					switch(TCH) {
						case 'W':
							return CollectData();
						}
				}
		}
	return 0;
	}

int PCL_Hook_CMD(const char cmdch)

	{
	if (was_data) was_data = 0;
	printf("!%c", cmdch);
	return 0;
	}

int PCL_Hook_TXT(const unsigned char db)

	/* the db byte could be part of the data stream for a PCL graphic
		command */

	{
	if (hex_mode)
		{
		if ((db >= CODE_SP)  && (db <= 126))
			ChrOut(db);
		else
			HexOut(db);
		}
	else
		{
		if ((db < CODE_SP)  || (db > 126))
			HexOut(db);
		else
			ChrOut(db);
		}
	return 0;
	}

int PCL_Hook_DAT(const unsigned char db)
	{
	return 0;
	}

int PCL_Hook_CTL(const unsigned char db)
	{
	switch (db)
		{
		case CODE_SP:
			ChrOut(db);
			break;
		default:
			HexOut(db);
		}
	return 0;
	}

char *UpperCase(char *sz)
	{
	int i;

	for (i = 0; i < strlen(sz); i++) sz[i] = toupper(sz[i]);

	return sz;
	}

int main(int argc, char *argv[])

	{
	int hInput;
	unsigned char databyte;
	char fs[128];

	MyAppHeader("PCL2TXT", VERSION, 1994);

	if (argc < 2)
		{
		printf("Syntax: PCL2TXT <file> [<stdout>]\n");
		printf("\n");
		printf("will dissasemble <file>, any HP PCL file, to <stdout> in a readable format.\n");
		printf("\n");
		FreewareMsg();
		printf("\n");
		printf("ANSI C compatible source and object libraries are available. Please call!\n");
		printf("The PCL Parser module allows you to incorporate HP PCL parsing into your own\n");
		printf("application(s). The file PCL2TXT.C demonstrates how the PCL Parser interfaces\n");
		printf("with an application.\n");
		exit(0);
		}

	strcpy(fs, argv[1]);
	UpperCase(fs);

	if ((hInput = open(fs, O_BINARY | O_RDONLY)) == -1)
		{
		printf("\n%s: File not found or not available.\n", fs);
		exit(0);
		}
	while (read(hInput, &databyte, 1) == 1) PCL_Parse(databyte);
	close(hInput);

	return 0;

	}
